const addCacheWrapper = (func) => {
    const cache = {};
    return function (...rest) {
        const key = JSON.stringify(rest);
        if (cache[key])
            return cache[key];
        cache[key] = func(...rest);
        return cache[key];
    };
};
const genSetRefObjFunc = (obj) => {
    return (key, value) => {
        if (typeof key === "string") {
            if (typeof value === "undefined")
                Reflect.deleteProperty(obj.current, key);
            else
                obj.current = Object.assign(Object.assign({}, obj.current), { [key]: value });
        }
        else if (typeof key === "function")
            obj.current = key(obj.current);
        else
            obj.current = Object.assign(Object.assign({}, obj.current), key);
    };
};
const genSetObjFunc = (obj) => {
    return (key, value) => {
        if (typeof key === "string") {
            if (typeof value === "undefined")
                Reflect.deleteProperty(obj, key);
            else
                obj[key] = value;
        }
        else if (typeof key === "function")
            key(obj);
        else
            Object.entries(key).map(([k, v]) => (obj[k] = v));
    };
};
const parseJson = (json) => {
    let res = null;
    if (!json)
        return res;
    try {
        res = JSON.parse(json);
    }
    catch (e) {
        console.error(e);
    }
    return res;
};
const getDom = (id, className) => {
    let dom = document.querySelector(`#${id}`);
    if (!dom) {
        dom = document.createElement("div");
        dom.id = id;
        document.body.appendChild(dom);
    }
    className && dom.classList.add(className);
    return dom;
};
// https://juejin.cn/post/7140558929750130719
const abortablePromise = (promise) => {
    const abortController = new AbortController();
    const { signal } = abortController;
    return {
        promise: new Promise((resolve, reject) => {
            signal.addEventListener("abort", () => reject(signal === null || signal === void 0 ? void 0 : signal.reason));
            promise.then(resolve).catch(reject);
        }),
        abortController,
    };
};
export default { addCacheWrapper, abortablePromise, genSetRefObjFunc, genSetObjFunc, parseJson, getDom };
