import { isNil } from "lodash";
class ListenValueHelper {
    constructor() {
        this.valueMap = new Proxy({}, {
            set: (obj, key, value) => {
                obj[key] = value;
                if (!isNil(value))
                    this.doResolveFunc(key);
                return true;
            },
        });
        this.resolveRejectMap = {};
        this.doResolveFunc = (key) => {
            const resolveRejectObj = this.resolveRejectMap[key];
            if (!resolveRejectObj)
                return;
            resolveRejectObj.resolveFuncList.forEach(item => item === null || item === void 0 ? void 0 : item(this.getValue(key)));
            resolveRejectObj.rejectTimerList.forEach(item => item && clearTimeout(item));
            resolveRejectObj.resolveFuncList = [];
            resolveRejectObj.rejectTimerList = [];
        };
        this.addListener = (key, timeout, signal) => {
            if (!this.resolveRejectMap[key])
                this.resolveRejectMap[key] = { resolveFuncList: [], rejectTimerList: [] };
            return new Promise((resolve, reject) => {
                signal === null || signal === void 0 ? void 0 : signal.throwIfAborted();
                signal === null || signal === void 0 ? void 0 : signal.addEventListener("abort", () => reject(signal === null || signal === void 0 ? void 0 : signal.reason));
                if (!isNil(this.valueMap[key]))
                    resolve(this.valueMap[key]);
                else {
                    const { resolveFuncList, rejectTimerList } = this.resolveRejectMap[key];
                    const resolveFunc = (value) => resolve(value);
                    resolveFuncList.push(resolveFunc);
                    // timeout
                    if (timeout && timeout > 0) {
                        const timer = setTimeout(() => {
                            reject(new Error("timeout"));
                            const index = resolveFuncList.findIndex(item => item === resolveFunc);
                            if (~index)
                                resolveFuncList[index] = null;
                        }, timeout);
                        rejectTimerList.push(timer);
                    }
                }
            });
        };
        this.getValue = (key) => this.valueMap[key];
        this.setValue = (key, value) => (this.valueMap[key] = value);
    }
}
export default ListenValueHelper;
